local Menu = require("Menu")
--------------------[-----------------------------[ Localising / Variables / Utilisation ]-----------------------------]--------------------
local Snake
local menu_font
local directionQueue
local foodPosition
local canvas
local BackgroundX, BackgroundY, Background

local Game = {}

--- [ Timer ] ---
local timer = 0

--- [ Debug ] ---
local Debug = 0

-------------------{{{{{{{{ notes }}}}}}}}-------------------
--[[

--
1 in gridXCount/gridYCount is equal to 8 squares of the background
so if gridYCount and gridXCount is equal to 10 then there would 6400 squares in total across the screen
that means alot of fucking math - Great T_T!
--

-- 11.5 is equal to love.graphics.getWidth()/X squares found in screen

--]]



-----------------------------[-----------------------------[ Main ]-----------------------------]-----------------------------

local function moveFood()
    local possibleFoodPositions = {}

    for foodX = 1, Menu.gridXCount do
        for foodY = 1, Menu.gridYCount do
            local possible = true

            for segmentIndex, segment in ipairs(Snake.Segments) do
                if foodX == segment.x and foodY == segment.y then
                    possible = false
                end
            end

            if possible then
                table.insert(possibleFoodPositions, {x = foodX, y = foodY})
            end
        end
    end

    foodPosition = possibleFoodPositions[love.math.random(#possibleFoodPositions)]
end


---[-[-[-[-[-[-[-[*** [ love.load() ] ***]-]-]-]-]-]-]-]---

function Game:load()
    menu_font = love.graphics.newFont('PixelFJVerdana12pt_0.ttf', 20)
    love.graphics.setFont(menu_font)

    canvas = love.graphics.newCanvas(love.graphics.getHeight(), love.graphics.getWidth())

    directionQueue = {'right'}

    Snake = {x = 100,
             y = 500,
             w = 20,
             h = 20,
             direction = "right",
             speed = 200,
             cellSize = 15,
             Alive = true,
             R = 0,
             G = 255,
             B = 0,
             Segments = {
             {x = 3, y = 1},
             {x = 2, y = 1},
             {x = 1, y = 1}
             }
             }

    Background = {
        Segments = {
             {x = 1, y = 1},
             {x = 2, y = 1},
             {x = 3, y = 1},
             {x = 4, y = 1},
             {x = 5, y = 1},
             {x = 6, y = 1},
             {x = 7, y = 1},
             {x = 8, y = 1},
             {x = 9, y = 1},
             {x = 10, y = 1}
             }
    }


    foodPosition = {
        x = love.math.random(1, Menu.gridXCount),
        y = love.math.random(1, Menu.gridYCount)
    }

    moveFood()

    BackgroundX = love.graphics.getWidth()/15
    BackgroundY = love.graphics.getHeight()/15

    if Menu.Active == 1 then
        Menu:load()
    end

--[[
    map = {}
    for x=0, Menu.gridXCount*8 do
        map[x] = {}
        for y=0, Menu.gridYCount*8 do
            map[x][y] = {
                x = x * Snake.cellSize,
                y = y * Snake.cellSize
              }
        end
    end

    for x=0, Menu.gridXCount*8 do
        for y=0, Menu.gridYCount*8 do
            table.insert(Background.Segments, math.floor(map[x][y].x), math.floor(map[x][y].y))
        end
    end
--]]

end

---[-[-[-[-[-[-[-[*** [ love.update(dt) ] ***]-]-]-]-]-]-]-]---

function Game:update(dt)
    if Menu.Active == 1 then
        Menu:update()
    else
        --- [ Snake Updating ] ---

        timer = timer + dt
        if Snake.Alive == true then
            local timerLimit = 0.15
            if timer >= timerLimit then
                timer = timer - timerLimit

                local nextXPosition = Snake.Segments[1].x
                local nextYPosition = Snake.Segments[1].y

                --- [ Cleanup ] ---
                if #directionQueue > 1 then
                    table.remove(directionQueue, 1)
                end

                --- [ Key handling ] ---
                if directionQueue[1] == 'right' then
                    nextXPosition = nextXPosition + 1
                elseif directionQueue[1] == 'left' then
                    nextXPosition = nextXPosition - 1
                elseif directionQueue[1] == 'down' then
                    nextYPosition = nextYPosition + 1
                elseif directionQueue[1] == 'up' then
                    nextYPosition = nextYPosition - 1
                end

                --- [ Collision check ] ---
                for segmentIndex, segment in ipairs(Snake.Segments) do
                    if segmentIndex ~= #Snake.Segments
                    and nextXPosition == segment.x
                    and nextYPosition == segment.y then
                        Snake.Alive = false
                    end
                end

                if nextYPosition > (39) then
                    Snake.Alive = false
                end
                if nextXPosition > (52) then
                    Snake.Alive = false
                end
                if nextYPosition < 1 then
                    Snake.Alive = false
                end
                if nextXPosition < 1 then
                    Snake.Alive = false
                end

                --- [ Tail Manager ] ---
                table.insert(Snake.Segments, 1, {x = nextXPosition, y = nextYPosition})
                table.remove(Snake.Segments)
                table.insert(Snake.Segments, 1, {x = nextXPosition, y = nextYPosition})


                if Snake.Segments[1].x == foodPosition.x and Snake.Segments[1].y == foodPosition.y then
                    table.insert(Snake.Segments, 1, {x = nextXPosition, y = nextYPosition})
                    moveFood()
                else
                    table.remove(Snake.Segments)
                end


                --- [ Background ] ---

            end
            --- [ Reset/ Game over ] ---
        elseif timer >= 2 then
            Menu.Active = 1
            love.load()
        end

    end
end

---[-[-[-[-[-[-[-[*** [ love.draw() ] ***]-]-]-]-]-]-]-]---

function Game:draw()
    if Menu.Active == 1 then
        Menu:draw()
    else

        --- [ Cell function ] ---
        local function drawCell(x, y)
            love.graphics.rectangle(
                    'fill',
                    (x - 1) * Snake.cellSize,
                    (y - 1) * Snake.cellSize,
                    Snake.cellSize - 1,
                    Snake.cellSize - 1
            )
        end

        --- [ Create Background ] ---
        --[[for segmentIndex, segment in ipairs(Background.Segments) do
            if segmentIndex ~= Menu.gridXCount*8 then
                love.graphics.setColor(255, 0, 0)
                drawCell(segment.x, segment.y)
                table.insert(Background.Segments, 1, {x = segment.x, y =  segment.y})
            end
        end--]]


        --- [ Bounding Box ] ---
        love.graphics.setColor(255, .28, .28)
        love.graphics.rectangle(
                'line',
                0,
                0,
                love.graphics.getWidth(),
                love.graphics.getHeight()
        )

        --- [ Snake color ] ---
        for segmentIndex, segment in ipairs(Snake.Segments) do
            if Snake.Alive then
                love.graphics.setColor(Snake.R, Snake.G, Snake.B)
            else
                love.graphics.setColor(.5, .5, .5)
            end
            drawCell(segment.x, segment.y)
        end

        love.graphics.setColor(1, .3, .3)
        drawCell(foodPosition.x, foodPosition.y)


        --- [ Debug ] ---
        if Debug == 1 then
        for directionIndex, direction in ipairs(directionQueue) do
            love.graphics.setColor(1, 1, 1)
            love.graphics.print('directionQueue['..directionIndex..']: '..direction, 15, 15 * directionIndex)
        end
        else
            love.graphics.print("", 15, 15)
        end
    end
end

---[-[-[-[-[-[-[-[*** [ love.keypressed(key) ] ***]-]-]-]-]-]-]-]---

function Game:keypressed(key)
    if Menu.Active == 1 then
        Menu.keypressed(key)
    else

        --- [ Key presses ] ---

        if key == 'right'
                and directionQueue[#directionQueue] ~= 'right'
                and directionQueue[#directionQueue] ~= 'left' then
            table.insert(directionQueue, 'right')

        elseif key == 'left'
                and directionQueue[#directionQueue] ~= 'left'
                and directionQueue[#directionQueue] ~= 'right' then
            table.insert(directionQueue, 'left')

        elseif key == 'up'
                and directionQueue[#directionQueue] ~= 'up'
                and directionQueue[#directionQueue] ~= 'down' then
            table.insert(directionQueue, 'up')

        elseif key == 'down'
                and directionQueue[#directionQueue] ~= 'down'
                and directionQueue[#directionQueue] ~= 'up' then
            table.insert(directionQueue, 'down')

        elseif key == '`' then
            Debug = Debug + 1
            if Debug == 2 then
                Debug = 0
            end
        end
    end
end

return Game, Snake
















